/*------------------------------------------------------------------------------
* convbin.c : convert receiver binary log file to rinex obs/nav, sbas messages
*
*          Copyright (C) 2007 by T.TAKASU, All rights reserved.
*
* options : -DWINDOWS  use windows file path separator
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 22:13:04 $
* history : 2008/06/22 1.0 new
*-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "rtklib.h"
#include "rcv/novatel.h"
#include "rcv/ublox.h"
#include "rcv/ss2.h"
#include "rcv/crescent.h"
#include "rcv/garmin.h"

static const char rcsid[]="$Id: convbin.c,v 1.1 2008/07/17 22:13:04 ttaka Exp $";

#define PRGNAME	"convbin ver.2.1"

#ifdef WIN32
#define	PATHSEP		'\\'			/* file path separator */
#else
#define	PATHSEP		'/'				/* file path separator */
#endif

/* help text -----------------------------------------------------------------*/
static const char *help[]={
"",
" Synopsys",
"",
" convbin [-ts y/m/d h:m:s] [-te y/m/d h:m:s] [-ti tint] [-r rcv] [-d dir]",
"         [-o ofile] [-n nfile] [-s sfile] file",
"",
" Description",
"",
" Convert receiver binary log file to RINEX OBS/NAV and SBAS message file.",
" SBAS message file complies to RTKLIB SBAS messsage format. Support the",
" following receivers and binary messages.",
"",
" NovAtel OEMV/4       : RANGECMPB, RAWEPHB, IONUTCB, RAWWASSFRMB",
" u-blox LEA-4T/AEK-4T : RXMRAW, RXMSFRB",
" NovAtel Superstar II : ID#20, ID#21, ID#22, ID#23, ID#67",
" Hemisphere Crescent  : BIN80, BIN94, BIN95, BIN96",
" Garmin GPS-15        : measurement",
" SiRF Star II         : (currently not supported)",
"",
" Options [default]",
"",
"     file      input receiver binary log file",
"     -ts y/m/d h:m:s  start time [all]",
"     -te y/m/d h:m:s  end time [all]",
"     -ti tint  observation data interval (s) [all]",
"     -r rcv    receiver type (nov=OEMV/4,ubx=u-blox,ss2=SuperstarII,",
"               cres=Crescent,garm=Garmin,sirf=SiRF II)",
"     -d dir    output directory [same as input file]",
"     -o ofile  output RINEX OBS file    [<file>.obs]",
"     -n nfile  output RINEX NAV file    [<file>.nav]",
"     -s sfile  output SBAS message file [<file>.sbs]",
"",
" If receiver type is not specified, type is recognized by the input",
" file extention as follows.",
"     *.gps    NovAtel OEMV/4",
"     *.ubx    u-blox LEA-4T/AEK-4T",
"     *.log    NovAtel Superstar II",
"     *.bin    Hemisphere Crescent",
"     *.gar    Garmin GPS-15"
"     *.sir    SiRF Star II"
};
/* print help ----------------------------------------------------------------*/
static void printhelp(void)
{
    int i;
    for (i=0;i<sizeof(help)/sizeof(*help);i++) fprintf(stderr,"%s\n",help[i]);
    exit(0);
}
/* show message --------------------------------------------------------------*/
extern int showmsg(char *format, ...)
{
    va_list arg;
    va_start(arg,format); vfprintf(stderr,format,arg); va_end(arg);
    fprintf(stderr,"\r");
    return 0;
}
/* main ----------------------------------------------------------------------*/
static int convbin(gtime_t ts, gtime_t te, double tint, char **file, int type,
                   const rnxopt_t *rnxopt, char *dir)
{
	char *rcv[]={"nov","ubx","ss2","cres","garm","sirf"};
	char ofile[1024],nfile[1024],sfile[1024],work[1024],*p;
	
	if (file[1]) strcpy(ofile,file[1]);
	else {
		strcpy(ofile,file[0]);
		if ((p=strrchr(ofile,'.'))) strcpy(p,".obs"); else strcat(ofile,".obs");
	}
	if (file[2]) strcpy(nfile,file[2]);
	else {
		strcpy(nfile,file[0]);
		if ((p=strrchr(nfile,'.'))) strcpy(p,".nav"); else strcat(nfile,".nav");
	}
	if (file[3]) strcpy(sfile,file[3]);
	else {
		strcpy(sfile,file[0]);
		if ((p=strrchr(sfile,'.'))) strcpy(p,".sbs"); else strcat(sfile,".sbs");
	}
	if (dir&&*ofile) {
		if ((p=strrchr(ofile,PATHSEP))) strcpy(work,p+1); else strcpy(work,ofile);
		sprintf(ofile,"%s%c%s",dir,PATHSEP,work);
	}
	if (dir&&*nfile) {
		if ((p=strrchr(nfile,PATHSEP))) strcpy(work,p+1); else strcpy(work,ofile);
		sprintf(nfile,"%s%c%s",dir,PATHSEP,work);
	}
	if (dir&&*sfile) {
		if ((p=strrchr(sfile,PATHSEP))) strcpy(work,p+1); else strcpy(work,ofile);
		sprintf(sfile,"%s%c%s",dir,PATHSEP,work);
	}
	fprintf(stderr,"input file: %s (%s)\n",file[0],rcv[type-1]);
	if (*ofile) fprintf(stderr,"rinex obs : %s\n",ofile);
	if (*nfile) fprintf(stderr,"rinex nav : %s\n",nfile);
	if (*sfile) fprintf(stderr,"sbas msg  : %s\n",sfile);
	if (!convlog(file[0],type,ofile,nfile,sfile,ts,te,tint,rnxopt,0)) {
		fprintf(stderr,"receiver log conversion error\n");
		return -1;
	}
	fprintf(stderr,"\n");
	return 0;
}
/* main ----------------------------------------------------------------------*/
int main(int argc, char **argv)
{
	rnxopt_t rnxopt={{0}};
	double tint=0.0,eps[]={1980,1,1,0,0,0},epe[]={2037,12,31,0,0,0};
	int i,type=0;
	char *rcv=NULL,*dir=NULL,*file[4]={0},*p;
	gtime_t ts={0},te={0};

	for (i=1;i<argc;i++) {
		if (!strcmp(argv[i],"-ts")&&i+2<argc) {
			sscanf(argv[++i],"%lf/%lf/%lf",eps,eps+1,eps+2);
			sscanf(argv[++i],"%lf:%lf:%lf",eps+3,eps+4,eps+5);
			ts=epoch2time(eps);
		}
		else if (!strcmp(argv[i],"-te")&&i+2<argc) {
			sscanf(argv[++i],"%lf/%lf/%lf",epe,epe+1,epe+2);
			sscanf(argv[++i],"%lf:%lf:%lf",epe+3,epe+4,epe+5);
			te=epoch2time(epe);
		}
		else if (!strcmp(argv[i],"-ti")&&i+1<argc) tint=atof(argv[++i]);
		else if (!strcmp(argv[i],"-r")&&i+1<argc) rcv=argv[++i];
		else if (!strcmp(argv[i],"-d")&&i+1<argc) dir=argv[++i];
		else if (!strcmp(argv[i],"-o")&&i+1<argc) file[1]=argv[++i];
		else if (!strcmp(argv[i],"-n")&&i+1<argc) file[2]=argv[++i];
		else if (!strcmp(argv[i],"-s")&&i+1<argc) file[3]=argv[++i];
		else if (!strncmp(argv[i],"-",1)) printhelp();
		else file[0]=argv[i];
	}
	if (!file[0]) {
		fprintf(stderr,"no input receiver log file\n");
		return -1;
	}
	if (rcv) {
		if      (!strcmp(rcv,"nov"))  type=RCVLOG_NOV;
		else if (!strcmp(rcv,"ubx"))  type=RCVLOG_UBX;
		else if (!strcmp(rcv,"ss2"))  type=RCVLOG_SS2;
		else if (!strcmp(rcv,"cres")) type=RCVLOG_CRES;
		else if (!strcmp(rcv,"garm")) type=RCVLOG_GARM;
		else if (!strcmp(rcv,"sirf")) type=RCVLOG_SIRF;
	}
	else if ((p=strrchr(file[0],'.'))) {
		if      (!strcmp(p,".gps"))  type=RCVLOG_NOV;
		else if (!strcmp(p,".ubx"))  type=RCVLOG_UBX;
		else if (!strcmp(p,".log"))  type=RCVLOG_SS2;
		else if (!strcmp(p,".bin"))  type=RCVLOG_CRES;
		else if (!strcmp(p,".gar"))  type=RCVLOG_GARM;
		else if (!strcmp(p,".sir"))  type=RCVLOG_SIRF;
	}
	if (type==0) {
		fprintf(stderr,"receiver type can not be recognized\n");
		return -1;
	}
	strcpy(rnxopt.prog,PRGNAME);
	sprintf(rnxopt.comment[0],"log file: %-53.53s",file[0]);
	return convbin(ts,te,tint,file,type,&rnxopt,dir);
}
