/*------------------------------------------------------------------------------
* rtkpos.c : realtime kinematic positioning
*
*          Copyright (C) 2007 by T.TAKASU, All rights reserved.
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 21:48:06 $
* history : 2007/01/12 1.0 new
*           2007/03/13 1.1 add slip detection by LLI flag
*           2007/04/18 1.2 add antenna pcv correction
*                          change rtkpos argin
*           2008/07/18 1.3 refactored
*-----------------------------------------------------------------------------*/
#include "rtklib.h"

static const char rcsid[]="$Id: rtkpos.c,v 1.1 2008/07/17 21:48:06 ttaka Exp $";

/* constants/macros ----------------------------------------------------------*/

#define SAT(s)      ((s)<=MAXPRNGPS?(s):MAXPRNGPS+1+(s)-MINPRNSBS)
#define PRN(s)      ((s)<=MAXPRNGPS?(s):MINPRNSBS+1+(s)-MAXPRNGPS)
#define II(s,opt)   ((opt->mode!=5?3:0)+(s)-1)     /* state index of ionos */
#define IT(r,opt)   (II(1,opt)+(opt->estiono?MAXSAT:0)+(r)) /* state index of tropos */
#define NR(opt)     (IT(0,opt)+(opt->esttrop?2:0)) /* number of real prms */
#define IB(s,f,opt) (NR(opt)+MAXSAT*(f)+(s)-1)     /* state index of bias */

const static double fion[]={    /* ionospheric delay factor relative to P1 */
    1.0,FREQ1*FREQ1/FREQ2/FREQ2,FREQ1*FREQ1/FREQ5/FREQ5
};
/* types --------------------------------------------------------------------*/
typedef struct {                /* ambiguity control struct */
    gtime_t time;               /* time */
    int vsat[MAXSAT][NFREQ];    /* valid satellite flag */
    int slip[MAXSAT][NFREQ];    /* cycle-slip flag */
    int lock[MAXSAT][NFREQ];    /* lock counter of phase */
    int outc[MAXSAT][NFREQ];    /* observation outage counter for phase */
    double gf[MAXSAT];          /* previous geometry-free phase (m) */
} ambctr_t;

/* single-differenced observable (f=0:L1,1:L2,2:C1,3:P2) ---------------------*/
static double sdobs(const obsd_t *obs, int i, int j, int f)
{
    double pi=f<NFREQ?lam[f]*obs[i].L[f]:obs[i].P[f-NFREQ];
    double pj=f<NFREQ?lam[f]*obs[j].L[f]:obs[j].P[f-NFREQ];
    return pi==0.0||pj==0.0?0.0:pi-pj;
}
/* single-differenced geometry-free linear combination of phase --------------*/
static double gfobs(const obsd_t *obs, int i, int j)
{
    double pi=sdobs(obs,i,j,0),pj=sdobs(obs,i,j,1);
    return pi==0.0||pj==0.0?0.0:pi-pj;
}
/* single-differenced measurement error variance (f=0:L1,1:L2,2:C1,3:P2) -----*/
static double varerr(double el, double bl, double dt, int f, const prcopt_t *opt)
{
    double a=opt->err[1],b=opt->err[2]/sin(el),c=opt->err[3]*bl/1E4;
    double d=CLIGHT*opt->sclkstab*dt;
    return 2.0*(a*a+b*b+c*c)*(f<opt->nf?1.0:opt->err[0]*opt->err[0])+d*d;
}
/* baseline length -----------------------------------------------------------*/
static double baseline(const double *ru, const double *rb)
{
    double dr[3];
    dr[0]=ru[0]-rb[0]; dr[1]=ru[1]-rb[1]; dr[2]=ru[2]-rb[2];
    return norm(dr,3);
}
/* initialize state and covariance -------------------------------------------*/
static void initx(double *x, double *P, int nx, double xi, double var, int i)
{
    int j;
    x[i]=xi; for (j=0;j<nx;j++) P[i+j*nx]=P[j+i*nx]=i==j?var:0.0;
}
/* select common satellites between rover and reference station --------------*/
static int selsat(const obsd_t *obs, double *azel, int nu, int nr,
                  const prcopt_t *opt, int *sat, int *iu, int *ir)
{
    int i,j,k=0;
    
    trace(3,"selsat : nu=%d nr=%d\n",nu,nr);
    
    for (i=0,j=nu;i<nu&&j<nu+nr;i++,j++) {
        if      (obs[i].sat<obs[j].sat) j--;
        else if (obs[i].sat>obs[j].sat) i--;
        else if (azel[1+j*2]>=opt->elmin&& /* elevation at base station */
                 (obs[i].SNR[0]==0||opt->snrmin<=obs[i].SNR[0])&&
                 (obs[j].SNR[0]==0||opt->snrmin<=obs[j].SNR[0])) {
            sat[k]=SAT(obs[i].sat); iu[k]=i; ir[k++]=j;
            trace(4,"(%2d) PRN%3d iu=%2d ir=%2d\n",k-1,obs[i].sat,i,j);
        }
    }
    return k;
}
/* temporal update of states -------------------------------------------------*/
static void udstate(double *x, double *P, int nx, const obsd_t *obs,
                    const int *sat, const int *iu, const int *ir, int ns,
                    double bl, ambctr_t *ambc, const prcopt_t *opt)
{
    gtime_t time=obs[0].time;
    double dt,g0,g1,cp,pr,bias[MAXSAT]={0},offset;
    int i,j,k,f,reset;
    
    trace(3,"udstate: ns=%d\n",ns);
    
    dt=ambc->time.time==0?0.0:fabs(timediff(time,ambc->time)); ambc->time=time;
    
    for (i=0;i<ns&&opt->mode>=2;i++) {
        for (f=0;f<opt->nf;f++) { /* detect slip by LLI */
            ambc->slip[sat[i]-1][f]=obs[iu[i]].LLI[f]|obs[ir[i]].LLI[f];
            if (f>=1) ambc->slip[sat[i]-1][f]&=~1; /* clear L2 LLI flag */
            if (ambc->slip[sat[i]-1][f]) {
                trace(2,"udstate: slip detected (PRN%2d L%d LLI=%x %x)\n",
                      PRN(sat[i]),f+1,obs[iu[i]].LLI[f],obs[ir[i]].LLI[f]);
            }
        }
        if (opt->nf>=2&&(g1=gfobs(obs,iu[i],ir[i]))!=0.0) {
            g0=ambc->gf[sat[i]-1]; ambc->gf[sat[i]-1]=g1;
            if (g0!=0.0&&fabs(g1-g0)>opt->thresslip) { /* detect slip by GF jump */
                trace(2,"udstate: slip detected (PRN%2d GF=%.3f %.3f)\n",
                      PRN(sat[i]),g0,g1);
                for (f=0;f<opt->nf;f++) ambc->slip[sat[i]-1][f]|=1;
            }
        }
    }
    for (f=0;f<opt->nf&&opt->mode>=2;f++) {
        /* reset phase-bias if instantaneous AR or expire obs outage counter */
        for (i=1;i<=MAXSAT;i++) {
            reset=++ambc->outc[i-1][f]>opt->maxout;
            if (opt->modear==1&&x[IB(i,f,opt)]!=0.0) {
                initx(x,P,nx,0.0,0.0,IB(i,f,opt));
            }
            else if (reset&&x[IB(i,f,opt)]!=0.0) {
                initx(x,P,nx,0.0,0.0,IB(i,f,opt));
                trace(2,"udstate: obs outage counter overflow (PRN%2d L%d n=%d)\n",
                      PRN(i),f+1,ambc->outc[i-1][f]);
            }
            if (opt->modear!=1&&reset) {
                ambc->lock[i-1][f]=-opt->minlock;
            }
        }
        for (i=0;i<ns;i++) { /* reset phase-bias if slip */
            k=IB(sat[i],f,opt); P[k+k*nx]+=opt->prn[0]*opt->prn[0]*dt;
            if (opt->modear==1||!(ambc->slip[sat[i]-1][f]&1)) continue;
            x[k]=0.0; ambc->lock[sat[i]-1][f]=-opt->minlock;
        }
        for (i=j=0,offset=0.0;i<ns;i++) { /* estimate approx phase-bias */
            cp=sdobs(obs,iu[i],ir[i],f); pr=sdobs(obs,iu[i],ir[i],NFREQ);
            if (cp!=0.0&&pr!=0.0) bias[i]=(cp-pr)/lam[f]; 
            if (x[IB(sat[i],f,opt)]!=0.0) {offset+=bias[i]-x[IB(sat[i],f,opt)]; j++;}
        }
        if (j>0) { /* assure phase-code coherency */
            for (i=1;i<=MAXSAT;i++) if (x[IB(i,f,opt)]!=0.0) x[IB(i,f,opt)]+=offset/j;
        }
        for (i=0;i<ns;i++) { /* set initial phase-bias */
            if (bias[i]==0.0||x[IB(sat[i],f,opt)]!=0.0) continue;
            initx(x,P,nx,bias[i],opt->std[0]*opt->std[0],IB(sat[i],f,opt));
        }
    }
    if (opt->estiono) { /* reset/update ionosphere parameter */
        for (i=1;i<=MAXSAT;i++) {
            if (x[II(i,opt)]!=0.0&&ambc->outc[i-1][0]>60&&ambc->outc[i-1][1]>60)
                x[II(i,opt)]=0.0;
        }
        for (i=0;i<ns;i++) {
            k=II(sat[i],opt);
            if (x[k]==0.0) initx(x,P,nx,1E-6,opt->std[1]*opt->std[1]*bl*bl/1E8,k);
            else P[k+k*nx]+=opt->prn[1]*opt->prn[1]*dt*bl*bl/1E8;
        }
    }
    if (opt->esttrop) { /* reset/update tropospheric parameter */
        for (i=0;i<2;i++) {
            k=IT(i,opt);
            if (x[k]==0.0) initx(x,P,nx,1E-6,opt->std[2]*opt->std[2],k);
            else P[k+k*nx]+=opt->prn[2]*opt->prn[2]*dt;
        }
    }
}
/* undifferenced phase/code residuals ----------------------------------------*/
static void zdres(const obsd_t *obs, int n, const double *rs, const double *dts,
                  const double *rr, const prcopt_t *opt, const pcv_t *pcv,
                  double *y, double *e, double *azel)
{
    double r,pos[3],dr[NFREQ]={0};
    int i,j;
    
    trace(3,"zdres  : n=%d\n",n);
    
    for (i=0;i<n*opt->nf*2;i++) y[i]=0.0;
    ecef2pos(rr,pos);
    
    for (i=0;i<n;i++) {
        if (SAT(obs[i].sat)>MAXSAT) continue;
        if ((r=geodist(rs+i*3,rr,e+i*3))<=0.0) continue;
        if (satazel(pos,e+i*3,azel+i*2)<opt->elmin) continue;
        
        /* satellite clock-bias and troposphere delay model */
        r+=-CLIGHT*dts[i]+tropmodel(pos,azel+i*2,0.5);
        
        /* antenna phase center correction */
        if (pcv) antmodel(pcv,azel+i*2,dr);
        
        for (j=0;j<opt->nf;j++) { /* observable - pseudorange */
            if (obs[i].L[j]!=0.0) y[j+i*opt->nf*2]=lam[j]*obs[i].L[j]-r-dr[j];
            if (obs[i].P[j]!=0.0) y[j+i*opt->nf*2+opt->nf]=obs[i].P[j]-r-dr[j];
        }
    }
    for (i=0;i<n;i++) {
        trace(4,"PRN%3d %13.3f %13.3f %13.3f %13.10f %6.1f %5.1f\n",
              obs[i].sat,rs[i*3],rs[1+i*3],rs[2+i*3],dts[i],azel[i*2]*R2D,
              azel[1+i*2]*R2D);
    }
    trace(4,"y=\n"); tracemat(4,y,opt->nf*2,n,13,3);
}
/* double-differenced phase/code residuals -----------------------------------*/
static int ddres(gtime_t time, double dt, double *x, int nx, const int *sat,
                 double *y, double *e, double *azel, const int *iu,
                 const int *ir, int ns, const double *ru, const double *rb,
                 const prcopt_t *opt, double *v, double *H, double *R,
                 ambctr_t *ambc)
{
    double bl,posu[3],posr[3],didxi=0.0,didxj=0.0,*im,*tmu,*tmr,*Ri,*Rj,*Hi;
    int i,j,k,f,nv=0,ny=ns*opt->nf*2,n[NFREQ*2]={0};
    
    trace(3,"ddres  : dt=%.1f nx=%d ns=%d\n",dt,nx,ns);
    
    im=mat(ns,1); tmu=mat(ns,1); tmr=mat(ns,1); Ri=mat(ny,1); Rj=mat(ny,1);
    
    bl=baseline(ru,rb); ecef2pos(ru,posu); ecef2pos(rb,posr);
    
    for (i=0;i<ns;i++) {
        if (opt->estiono) {
            im[i]=(ionmapf(posu,azel+iu[i]*2)+ionmapf(posr,azel+ir[i]*2))/2.0;
        }
        if (opt->esttrop) {
            tropmapf(time,posu,azel+iu[i]*2,tmu+i);
            tropmapf(time,posr,azel+ir[i]*2,tmr+i);
        }
    }
    for (f=opt->mode>=2?0:opt->nf;f<opt->nf*2;f++) {
        
        /* make double-difference */
        for (i=j=0;j<ns;j++) {
            if (y[f+iu[j]*opt->nf*2]==0.0||y[f+ir[j]*opt->nf*2]==0.0) continue;
            if (azel[1+iu[j]*2]>=azel[1+iu[i]*2]) i=j; /* highest elevation */
        }
        for (j=0;j<ns;j++) {
            if (i==j) continue;
            if (y[f+iu[j]*opt->nf*2]==0.0||y[f+ir[j]*opt->nf*2]==0.0) continue;
              
            /* double-differenced residual */
            v[nv]=(y[f+iu[i]*opt->nf*2]-y[f+ir[i]*opt->nf*2])-
                  (y[f+iu[j]*opt->nf*2]-y[f+ir[j]*opt->nf*2]);
            
            if (opt->estiono) {
                didxi=(f<opt->nf?-1.0:1.0)*fion[f%opt->nf]*im[i];
                didxj=(f<opt->nf?-1.0:1.0)*fion[f%opt->nf]*im[j];
                v[nv]-=didxi*x[II(sat[i],opt)]-didxj*x[II(sat[j],opt)];
            }
            if (opt->esttrop) {
                v[nv]-=(tmu[i]-tmu[j])*x[IT(0,opt)]-(tmr[i]-tmr[j])*x[IT(1,opt)];
            }
            if (f<opt->nf) {
                v[nv]-=lam[f]*(x[IB(sat[i],f,opt)]-x[IB(sat[j],f,opt)]);
            }
            if (fabs(v[nv])>opt->maxinno) {
                trace(1,"ddres  : outlier rejected (PRN%2d-%2d %s%d v=%.3f)\n",
                      PRN(sat[i]),PRN(sat[j]),f<opt->nf?"L":"P",f%opt->nf+1,v[nv]);
                continue;
            }
            if (H) { /* partial derivatives */
                Hi=H+nv*nx; for (k=0;k<3;k++) Hi[k]=-e[k+iu[i]*3]+e[k+iu[j]*3];
                if (opt->estiono) {
                    Hi[II(sat[i],opt)]= didxi;
                    Hi[II(sat[j],opt)]=-didxj;
                }
                if (opt->esttrop) {
                    Hi[IT(0,opt)]= (tmu[i]-tmu[j]);
                    Hi[IT(1,opt)]=-(tmr[i]-tmr[j]);
                }
                if (f<opt->nf) {
                    Hi[IB(sat[i],f,opt)]= lam[f];
                    Hi[IB(sat[j],f,opt)]=-lam[f];
                }
            }
            if (f<opt->nf) ambc->vsat[sat[i]-1][f]=ambc->vsat[sat[j]-1][f]=1;
            Ri[nv]=varerr(azel[1+iu[i]*2],bl,dt,f,opt);
            Rj[nv]=varerr(azel[1+iu[j]*2],bl,dt,f,opt);
            
            trace(4,"PRN%3d-%3d %s%d v=%13.3f R=%8.6f %8.6f\n",PRN(sat[i]),
                  PRN(sat[j]),f<opt->nf?"L":"P",f%opt->nf+1,v[nv],Ri[nv],Rj[nv]);
            nv++; n[f]++;
        }
    }
    if (H) {trace(5,"H=\n"); tracemat(5,H,nx,nv,7,4);}
    
    /* measurement error covariance */
    for (i=0;i<nv*nv;i++) R[i]=0.0;
    for (f=k=0;f<opt->nf*2;k+=n[f++]) {
        for (i=k;i<k+n[f];i++) {
            for (j=i;j<k+n[f];j++) R[i+j*nv]=R[j+i*nv]=Ri[i];
            R[i+i*nv]+=Rj[i];
        }
    }
    trace(5,"R=\n"); tracemat(5,R,nv,nv,8,6);
    
    free(im); free(tmu); free(tmr); free(Ri); free(Rj);
    return nv;
}
/* time-interpolation of residuals -------------------------------------------*/
static double intpres(gtime_t time, const obsd_t *obs, int n, const nav_t *nav,
                      const double *rb, const prcopt_t *opt, double *y)
{
    static obsd_t obsb[MAXOBS];
    static double yb[MAXOBS*NFREQ*2];
    static double rs[MAXOBS*3],dts[MAXOBS],e[MAXOBS*3],azel[MAXOBS*2];
    static int nb=0;
    double dt=timediff(time,obs[0].time),dtb,*p,*q;
    int i,j,k;
    
    trace(3,"intpres: n=%d dt=%.1f\n",n,dt);
    
    if (nb==0||fabs(dt)<DTTOL) {
        nb=n; for (i=0;i<n;i++) obsb[i]=obs[i];
        return dt;
    }
    dtb=timediff(time,obsb[0].time);
    if (fabs(dtb)>opt->maxtdiff*2.0||dtb==dt) return dt;
    
    satpos(time,obsb,nb,nav,rs,dts);
    zdres(obsb,nb,rs,dts,rb,opt,opt->pcvr,yb,e,azel);
    for (i=0;i<n;i++) {
        for (j=0;j<nb;j++) if (obsb[j].sat==obs[i].sat) break;
        if (j>=nb) continue;
        for (k=0,p=y+i*opt->nf*2,q=yb+j*opt->nf*2;k<opt->nf*2;k++,p++,q++) {
            if (*p==0.0||*q==0.0) *p=0.0; else *p=(dtb*(*p)-dt*(*q))/(dtb-dt);
        }
    }
    return fabs(dtb)>fabs(dt)?dtb:dt;
}
/* relative positioning ------------------------------------------------------*/
static int relpos(double *x, double *P, int nx, const obsd_t *obs, int nu,
                  int nr, const nav_t *nav, const sbs_t *sbs, ambctr_t *ambc,
                  const double *rb, const prcopt_t *opt)
{
    gtime_t time=obs[0].time;
    double *rs,*dts,*y,*e,*azel,*v,*H,*R,*xp,*Pp,dt,vv;
    int i,j,f,n=nu+nr,ns,ny,nv,sat[MAXSAT],iu[MAXSAT],ir[MAXSAT],info=-1;
    
    trace(3,"relpos : nx=%d nu=%d nr=%d\n",nx,nu,nr);
    
    dt=timediff(time,obs[nu].time);
    
    rs=mat(3,n); dts=mat(1,n); y=mat(opt->nf*2,n); e=mat(3,n); azel=zeros(2,n);
    
    for (i=0;i<MAXSAT;i++) for (j=0;j<NFREQ;j++) {
        ambc->vsat[i][j]=ambc->slip[i][j]=0;
    }
    /* satellite position/clock-bias */
    satpos(time,obs,n,nav,rs,dts);
    
    /* residuals of base station */
    zdres(obs+nu,nr,rs+nu*3,dts+nu,rb,opt,opt->pcvr,y+nu*opt->nf*2,e+nu*3,
          azel+nu*2);
    
    /* time-interpolation of residuals */
    if (opt->intpref) dt=intpres(time,obs+nu,nr,nav,rb,opt,y+nu*opt->nf*2);
    
    /* select common satellites */
    if ((ns=selsat(obs,azel,nu,nr,opt,sat,iu,ir))<=0) {
        trace(1,"relpos : no common satellite\n");
        free(rs); free(dts); free(y); free(e); free(azel);
        return -1;
    }
    /* temporal update of states */
    udstate(x,P,nx,obs,sat,iu,ir,ns,baseline(x,rb),ambc,opt);
    
    trace(4,"x(0)="); tracemat(4,x,1,NR(opt),13,4);
    
    ny=ns*opt->nf*2;
    xp=mat(nx,1); Pp=mat(nx,nx); v=mat(ny,1); H=zeros(nx,ny); R=mat(ny,ny);
    matcpy(xp,x,nx,1);
    
    for (i=0;i<opt->niter;i++) {
        matcpy(Pp,P,nx,nx);
        
        /* rover undifferenced residuals */
        zdres(obs,nu,rs,dts,xp,opt,opt->pcvu,y,e,azel);
        
        /* double-differenced residuals */
        if ((nv=ddres(time,dt,xp,nx,sat,y,e,azel,iu,ir,ns,xp,rb,opt,v,H,R,ambc))<1) {
            trace(1,"relpos : no double-differenced residual\n");
            info=-1; break;
        }
        /* kalman filter measurement update */
        if ((info=filter(xp,Pp,H,v,R,nx,nv))) {
            trace(1,"relpos : filter error (info=%d)\n",info);
            break;
        }
        trace(4,"x(%d)=",i+1); tracemat(4,xp,1,NR(opt),13,4);
    }
    if (!info) { /* validation by postfit-residual test */
        zdres(obs,nu,rs,dts,xp,opt,opt->pcvu,y,e,azel);
        nv=ddres(time,dt,xp,nx,sat,y,e,azel,iu,ir,ns,xp,rb,opt,v,NULL,R,ambc);
        if (nv>NR(opt)) {
            for (i=0,vv=0.0;i<nv;i++) vv+=v[i]*v[i]/R[i+i*nv];
            if (vv>chisqr[nv-NR(opt)-1]) info=-9;
            trace(!info?2:1,"relpos : validation %s (nv=%d nr=%d vv=%.2f cs=%.2f)\n",
                  !info?"ok":"error",nv,NR(opt),vv,chisqr[nv-NR(opt)-1]);
        }
    }
    if (!info) { /* update state, covariance and ambiguity control struct */
        matcpy(x,xp,nx,1); matcpy(P,Pp,nx,nx);
        
        for (i=0;i<ns;i++) for (f=0;f<opt->nf;f++) {
            if (!ambc->vsat[sat[i]-1][f]) continue;
            ambc->lock[sat[i]-1][f]++; ambc->outc[sat[i]-1][f]=0;
            if (azel[1+iu[i]*2]<opt->elmaskar) ambc->lock[sat[i]-1][f]=0;
        }
    }
    free(rs); free(dts); free(y); free(e); free(azel);
    free(xp); free(Pp); free(v); free(H); free(R);
    return info;
}
/* resolve integer ambiguity -------------------------------------------------*/
static int resamb(double *x, double *P, int nx, const prcopt_t *opt,
                  ambctr_t *ambc, double *rr, double *Qr)
{
    int i,j,k,f,ny,nr=NR(opt),nb=0,info,*indi,*indj;
    double *D,*DP,*y,*Qy,*b,*db,*Qb,*Qrb,*QQ,s[2];
    
    trace(3,"resamb : nx=%d\n",nx);
    
    D=zeros(nx,nx); indi=imat(MAXSAT*opt->nf,1); indj=imat(MAXSAT*opt->nf,1);
    
    /* single to double-difference transformation matrix (D') */
    for (i=0;i<nr;i++) D[i+i*nx]=1.0;
    for (f=0,k=nr;f<opt->nf;f++,k+=MAXSAT) {
        for (i=k;i<k+MAXSAT;i++) {
            if (x[i]!=0.0&&ambc->lock[i-k][f]>0&&!(ambc->slip[i-k][f]&2)) break;
        }
        for (j=i+1;j<k+MAXSAT;j++) {
            if (x[j]!=0.0&&ambc->lock[j-k][f]>0&&!(ambc->slip[j-k][f]&2)) {
                D[i+(nr+nb)*nx]=1.0; D[j+(nr+nb)*nx]=-1.0;
                indi[nb]=i; indj[nb++]=j;
            }
        }
    }
    if (nb<=0) {
        trace(1,"resamb : no valid double-difference\n");
        free(D); free(indi); free(indj);
        return -1;
    }
    ny=nr+nb; y=mat(ny,1); Qy=mat(ny,ny); DP=mat(ny,nx);
    b=mat(nb,2); db=mat(nb,1); Qb=mat(nb,nb); Qrb=mat(nr,nb); QQ=mat(nr,nb);
    
    trace(5,"D=\n"); tracemat(5,D,nx,ny,2,0);
    
    /* transform single to double-differenced phase-bias (y=D'*x, Qy=D'*P*D) */
    matmul("TN",ny,1,nx,1.0,D,x,0.0,y);
    matmul("TN",ny,nx,nx,1.0,D,P,0.0,DP);
    matmul("NN",ny,ny,nx,1.0,DP,D,0.0,Qy);
    
    /* phase-bias covariance (Qb) and real-parameters to bias covariance (Qrb) */
    for (i=0;i<nb;i++) for (j=0;j<nb;j++) Qb[i+j*nb]=Qy[nr+i+(nr+j)*ny];
    for (i=0;i<nr;i++) for (j=0;j<nb;j++) Qrb[i+j*nr]=Qy[i+(nr+j)*ny];
    
    trace(4,"N(0)="); tracemat(4,y+nr,1,nb,10,3);
    
    /* lambda/mlambda integer least-square estimation */
    if (!(info=lambda(nb,2,y+nr,Qb,b,s))) {
        
        trace(4,"N(1)="); tracemat(4,b,1,nb,10,3);
        trace(4,"N(2)="); tracemat(4,b+nb,1,nb,10,3);
        
        /* validation by popular ratio-test */
        if (s[0]<=0.0||s[1]/s[0]>=opt->thresar) {
            
            /* transform float to fixed solution (rr=rr-Qrb*Qb\(b0-b)) */
            for (i=0;i<nr;i++) {
                rr[i]=x[i]; for (j=0;j<nr;j++) Qr[i+j*nr]=P[i+j*nx];
            }
            for (i=0;i<nb;i++) y[nr+i]-=b[i];
            if (!(info=matinv(Qb,nb))) {
                matmul("NN",nb,1,nb,1.0,Qb,y+nr,0.0,db);
                matmul("NN",nr,1,nb,-1.0,Qrb,db,1.0,rr);
                
                /* covariance (Qr=Qr-Qrb*Qb^-1*Qrb') */
                matmul("NN",nr,nb,nb,1.0,Qrb,Qb,0.0,QQ);
                matmul("NT",nr,nr,nb,-1.0,QQ,Qrb,1.0,Qr);
            }
        }
        else info=-1; /* validation failed */
        
        trace(!info?2:1,"resamb : validation %s (nb=%d ratio=%.2f s=%.2f/%.2f)\n",
              !info?"ok":"error",nb,s[1]/s[0],s[0],s[1]);
    }
    else trace(1,"resamb : lambda error (info=%d)\n",info);
    
    free(D); free(indi); free(indj); free(y); free(Qy); free(DP);
    free(b); free(db); free(Qb); free(Qrb); free(QQ);
    return info; /* 0:ok */
}
/* relative positioning --------------------------------------------------------
* input observation data and navigation message, compute rover position by 
* relative positioning and output results. if callback input() return 0, exit
* the function.
* args   : infunc_t input   I   input callback function
*          outfunc_t output I   output callback function
*          prcopt_t  *opt      I   positioning options (see rtklib.h)
* return : none
* notes  : input and output callback shall comply the following prototypes.
*
*          int input(obsd_t *obs, nav_t *nav);
*            obsd_t **obs   O   observation data records for an epoch
*                               obs[i].rcv=1:rover,2:reference
*                               sorted by receiver and satellte
*            nav_t  **nav   O   navigation messages
*            obsc_t **obsc  O   observation corrections for an epoch
*            int    *nc     O   number of observation corrections
*            return             number of observation data records
*
*          void output(gtime_t t, sol_t sol, sol_t solf, int stat, int ns);
*            gtime_t t      I   time
*            sol_t sol,solf I   float, dgps or single solution, fixed solution
*            double *rb     I   reference station position (ecef) (m)
*            int    stat    I   status (1:fix,2:float,4:dgps,5:single)
*            int    ns      I   number of valid satellites
*-----------------------------------------------------------------------------*/
extern void rtkpos(infunc_t input, outfunc_t output, const prcopt_t *opt)
{
    ambctr_t ambc={{0}};
    obsd_t *obs;
    nav_t *nav;
    sbs_t *sbs;
    sol_t sol={0},solf={0};
    double *x,*P,*azel,tt,rr[3]={0},rb[3]={0},Qr[9],Qb[9],dtr[2];
    int i,j,n,nx,ny,nu,nr,ns,stat;
    char s[64];
    
    trace(3,"rtkpos : opt.mode=%d\n",opt->mode);
    
    n=NR(opt); nx=n+MAXSAT*opt->nf; sol.n=solf.n=n;
    x=zeros(nx,1); P=zeros(nx,nx); azel=mat(2,MAXOBS);
    sol.rr=mat(n,1); sol.Qr=mat(n,n); solf.rr=mat(n,1); solf.Qr=mat(n,n);
    
    /* input observation data and navigation messages for an epoch */
    while ((ny=input(&obs,&nav,&sbs))>0) {
        time2str(obs[0].time,s,3);
        trace(2,"rtkpos : time=%s nobs=%d nnav=%d nsbs=%d\n",s,ny,nav->n,sbs->n);
        trace(4,"obs=\n"); traceobs(4,obs,ny);
        
        /* count rover/base station observations */
        for (nu=0;nu<ny&&obs[nu].rcv==1;nu++) ;
        for (nr=0;nu+nr<ny&&obs[nu+nr].rcv==2;nr++) ;
        if (opt->mode>=1) {
            if (nu==0||nr==0) {
                trace(1,"rtkpos : number of obs error (nu=%d nr=%d)\n",nu,nr);
                continue;
            }
            if (fabs(tt=timediff(obs[0].time,obs[nu].time))>opt->maxtdiff) {
                trace(1,"rtkpos : age of differential error (tt=%.1f)\n",tt);
                continue;
            }
        }
        /* rover position by single-point positioning */
        if ((ns=pntpos(obs,nu,nav,opt->elmin,opt->snrmin,rr,Qr,dtr,azel))<=0) {
            trace(1,"rtkpos : initial rover position error\n");
            continue;
        }
        stat=5;
        for (i=0;i<3;i++) {
            sol.rr[i]=rr[i]; for (j=0;j<3;j++) sol.Qr[i+j*n]=Qr[i+j*3];
        }
        if (opt->mode>=1) {
            /* base station position */
            if (opt->mode!=4) for (i=0;i<3;i++) rb[i]=opt->rb[i];
            else if (pntpos(obs+nu,nr,nav,opt->elmin,opt->snrmin,rb,Qb,dtr+1,azel)<=0) {
                trace(1,"rtkpos : base station position error\n");
                continue;
            }
            /* first epoch or kinematic mode */
            if (norm(x,3)<=0.0||opt->mode<=2||opt->mode==4) {
                for (i=0;i<3;i++) {
                    x[i]=rr[i];
                    for (j=i;j<nx;j++) P[i+j*nx]=P[j+i*nx]=j<3?Qr[i+j*3]:0.0;
                }
            }
            /* rover position by relative positioning */
            if (!relpos(x,P,nx,obs,nu,nr,nav,sbs,&ambc,rb,opt)) {
                
                for (i=0;i<n;i++) {
                    sol.rr[i]=x[i]; for (j=0;j<n;j++) sol.Qr[i+j*n]=P[i+j*nx];
                }
                stat=opt->mode<=1?4:2; /* 1:fix,2:float,4:dgps,5:single */
                
                /* resolve integer ambiguity */
                if (opt->mode>=2&&opt->thresar>=1.0) {
                    if (!resamb(x,P,nx,opt,&ambc,solf.rr,solf.Qr)) stat=1;
                }
            }
            else continue;
        }
        /* output results */
        output(timeadd(obs[0].time,-dtr[0]),sol,solf,rb,stat,ns);
    }
    free(x); free(P); free(azel);
    free(sol.rr); free(sol.Qr); free(solf.rr); free(solf.Qr);
}
